#--
#
# Author:
#   Sebastian Tello, Aureliano Calvo
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--

from impact_manager.impact_db_manager import ImpactDBManager
from impact_manager.impact_com_manager import ImpactComManager
from impact_manager.entity_manager import EntityManager
from impact_manager.entities.module import Module

class Workspace:

    EVENT_OUTPUT = 1
    EVENT_LOGLINE = 2
    EVENT_PROGRESS_UPDATED = 3
    EVENT_TASK_INITIAL = 4
    EVENT_TASK_RUNNING = 5
    EVENT_TASK_FINISHED = 6
    EVENT_TASK_ABORTED = 7
    EVENT_TASK_STOPPED = 8
    EVENT_TASK_STOPPING = 9
    EVENT_CHECK_UPDATES_FINAL = 10
    EVENT_TASK_ERROR = 11
    EVENT_CUSTOM = 12

    def __init__(self, _id, name, passphrase, com):
        self.id = _id
        if name != "":
            self.name = name
        else:
            db = ImpactDBManager()
            query = "SELECT Name " \
                "FROM TWorkspace " \
                "WHERE Workspace_Id = %d " % (self.id)
            self.name = db.execute(query).value(0)

        self.passphrase = passphrase
        self.__com_handler = com
        try:
            self.__entity_mgr_com = self.__com_handler.getEntityMgr()
            self.__entity_mgr_event_handler = ImpactComManager \
                                                 .get_entity_mgr_event_handler()
            self.__entity_mgr_com.registerGeneralHandler(
                                      self.__entity_mgr_event_handler.wrap_me())
            self.__module_execution_com = self.__com_handler \
                                                           .getModuleExecution()
            self.__general_module_execution_event_handler = ImpactComManager \
                                   .get_general_module_execution_event_handler()

            self.__module_execution_com.registerGeneralHandler(
                        self.__general_module_execution_event_handler.wrap_me())

        except Exception, e:
            #IN CASE SOMEONE INSTANCES THIS CLASS WITHOUT ITS COM OBJECT
            print e
            pass

    def __del__(self):
        """This doesn't seems to release the workspace"""
        del self.__entity_mgr_com
        del self.__entity_mgr_event_handler
        del self.__module_execution_com
        del self.__com_handler

    def get_com_handler(self):
        """Returns this workspace COM object"""
        return self.__com_handler

    def get_module_execution_com(self):
        """Returns a COM object instance representing a Module execution object
        for this workspace"""
        return self.__module_execution_com


    def get_entity_mgr_event_handler(self):
        """Returns the event handler for entities of these workspace.
        Where you can subscribe to events of creation, edition, and deletion of
        entities"""
        return self.__entity_mgr_event_handler

    def get_entity_manager_com(self):
        """Returns a COM object instance representing a EntityMgr object for
        this workspace"""
        return self.__entity_mgr_com

    def get_property(self, property_name):
        """ Returns the value for the property as defined in the DB
        (TWorkspaceProperty)
        Returns None if the property does not exist.
        Examples of properties: 'UNICODE', 'NEVER_OPEN', 'FROM_SCHEDULER',
        'START', 'DEADLINE'... """
        db = ImpactDBManager()
        property_name = db.addslashes(property_name)

        query = "SELECT Value_Tx " \
                "FROM TWorkspaceProperty " \
                "WHERE Workspace_Id = %d " \
                "AND Name = '%s';" % (self.id, property_name)
        rs = db.execute(query)
        return rs.value(0)

    def has_property(self, property_name):
        """ Returns True if the property given for the current workspace exists
        in the DB (TWorkspaceProperty) If not, returns False """

        db = ImpactDBManager()
        property_name = db.addslashes(property_name)

        query = "SELECT Value_Tx " \
                "FROM TWorkspaceProperty " \
                "WHERE Workspace_Id = %d " \
                "AND Name = '%s';" % (self.id, property_name)
        rs = db.execute(query)
        return rs.num_rows() > 0

    def set_property(self, property_name, property_value):
        """ Sets a property for the current workspace. If the property already
        exists, it is updated, if not a new property is created. """
        db = ImpactDBManager()

        property_name = db.addslashes(property_name)
        property_value = db.addslashes(property_value)

        if self.has_property(property_name):
            query = "UPDATE TWorkspaceProperty " \
                    "SET Value_Tx = '%s' " \
                    "WHERE Workspace_Id = %d " \
                    "AND Name = '%s';" % (property_value, self.id,
                                                                  property_name)
        else:
            query = "INSERT INTO TWorkspaceProperty " \
                    "(Workspace_Id, Type_Nr, Name, Value_Tx) " \
                    "VALUES(%d, 0, '%s', '%s');" % (self.id, property_name,
                                                                 property_value)

        db.execute_non_select(query)
        return None


    def get_properties_as_dict(self):
        """ Returns all the properties defined for the current workspace in the
        DB (TWorkspaceProperty). A dictionary is returned with the property
        names as keys, and the Value_Tx column as values """
        db = ImpactDBManager()
        query = "SELECT Name, Value_Tx " \
                "FROM TWorkspaceProperty " \
                "WHERE Workspace_Id = %d;" % (self.id)
        rs = db.execute(query)
        ret = {}
        while not rs.EOF:
            ret[rs.value('Name')] = rs.value('Value_Tx')
            rs.next()
        return ret

    def get_tasks_list(self):
        """ Returns a list of tasks. Each position of the list contains another
        list with the following data:
            pos 0 = Task_Id
            pos 1 = ModuleName
            pos 2 = Started
            pos 3 = Finished
            pos 4 = Status"""

        db = ImpactDBManager()
        query = """SELECT Task_Id, ModuleName, Started, Finished, Status
                   FROM VTask
                   WHERE Workspace_Id = %d """  % (self.id)
        rs = db.execute(query)

        tasks = []
        while not rs.EOF:
            t_property = []
            t_property.append(rs.value('Task_Id'))
            t_property.append(rs.value('ModuleName'))
            t_property.append(rs.value('Started'))
            t_property.append(rs.value('Finished'))
            t_property.append(rs.value('Status'))
            tasks.append(t_property)
            rs.next()
        return tasks
    
    def get_task_log(self, task_id, tail_lines=0, detail='high'):
        """Returns the last tail_lines lines from the log in the detail level
        provided. If tail_lines is not provided (or equal or lower than zero)
        the whole log is returned.
        Possible detail levels are: 'high', 'medium, 'low'.
        If it is not provided 'high' is used.
        If the module hasn't been started yet, None is returned """

        log_level = 2 #high is default
        if detail.lower() == 'medium':
            log_level = 1
        elif detail.lower() == 'low':
            log_level = 0

        db = ImpactDBManager()

        limit = ""
        if tail_lines > 0:
            limit = "TOP %d" % (int(tail_lines))

        query = "SELECT %s Text_Tx " \
                "FROM TLogLine " \
                "WHERE Workspace_Id = %d " \
                "AND Task_Id = %d " \
                "AND TraceLevel <= %d " \
                "ORDER BY Line_Nr DESC" % (limit , self.id, task_id, log_level)
        rs = db.execute(query)

        ret = ""
        while not rs.EOF:
            line = rs.value(0)
            if line[-1] == '\x01':
                #line ending with \01. Should be concatenated with the following
                ret = "%s%s" % (line[:-1], ret)
            else:
                ret = "%s\n%s" % (line, ret)
            rs.next()
        return ret


    def get_task_parameters(self,task_id):
        """ Returns the parameters of a task of the workspace"""

        db = ImpactDBManager()
        query = """SELECT Name, Value_Tx, Type, Priority, Row_Id
                   FROM TParameter
                   WHERE Workspace_Id = %d
                   AND Task_Id = %d """  % (self.id, task_id)
        rs = db.execute(query)

        param_list = []
        while not rs.EOF:
            #pos 0: Name of the parameter
            #pos 1: Value of the parameter
            param = []
            param.append(rs.value('Name'))
            param.append(rs.value('Value_Tx'))
            param_list.append(param)
            rs.next()
        return param_list

    def get_task_output(self, task_id):
        """ Returns the task's XML output as stored in the database."""

        db = ImpactDBManager()
        query = "SELECT \"Output_XML\"" \
                "FROM \"dbo\".\"TOutput\"" \
                "WHERE Workspace_Id = %d and Task_Id = %d" % (self.id, task_id)

        rs = db.execute(query)
        ret = rs.value(0)
        if ret is None: ret = ""
        return ret

    def stop_task(self, task_id):
        """ Stops the task identified by task_id in this workspace"""
        self.__module_execution_com.stopModule(int(task_id))


    def subscribe_to_tasks_events(self, event_types, function,
                                                        *additional_parameters):
        """Subscribe a function to tasks events triggered in this workspace.

        event_types: is a list of event types you want to be notified about
            e.g. [Module.EVENT_ENTITY_CREATED, Module.EVENT_ENTITY_DELETED]
        function: the function that will be called when the event occurs
        additional_parameters: When the callback function is called, it will
            provide a dictionary parameter containing information about the
             event, you can give any other parameters you want to be passed
             besides the data dictionary

        The data dictionary passed back to the subscribed functions contains:
            'EVENT_TIME': float: number of seconds since the epoc
            'EVENT_TYPE': int: See EVENT_* constants in this class
            'WORKSPACE_ID': int: workspace_id,
            'TASK_ID': int: task_id,
            [additional values depending on the event]

        Note: Keep in mind that you won't get event notifications generated by
        tasks in other workspaces. Example:

        def myfunc(greeting, data):
            print greeting
            print "your event data is:", data
        my_wsp.subscribe_to_tasks_events([Workspace.EVENT_TASK_INITIAL],
                                        myfunc,
                                        "Hi There!")
        mod.run()
        """
        self.__general_module_execution_event_handler.subscribe(event_types,
                    function, additional_parameters)


    def task(self, task_id):
        """Returns a Module instance of task already run"""

        db = ImpactDBManager()
        query = """ SELECT T.ModuleName, E.Name as AgentName
                    FROM TTask T
                    JOIN TEntity E
                    ON T.Workspace_Id = E.Workspace_Id
                    AND T.Agent_Id = E.Id
                    WHERE T.Workspace_Id = %d
                    AND T.Task_Id = %d;""" % (self.id, task_id)

        rs = db.execute(query)
        if not rs.num_rows():
            raise AttributeError("Invalid task_id for this workspace")

        task_data = rs.get_row_as_dict()
        mod = self.module(task_data['ModuleName'], task_data['AgentName'])
        mod.set_task_id(task_id)
        return mod

    @property
    def entity_manager(self):
        """Returns and EntityManager for this workspace"""
        return EntityManager(self)

    def module(self, name, agent='localagent'):
        """Returns a module instance to be run in this workspace"""
        return Module(self, name, agent)

    def __repr__(self):
        return "<Workspace:%d:%s>" % (self.id, self.name)