#--
# $Revision: 818 $
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--


import _winreg

class ImpactConfigurationManager:

    @staticmethod
    def get_registry_path():
        """Returns the path to Impact in the registry"""
        subkey = r"SOFTWARE\CORE Security Technologies"
        key = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, subkey)
        index = 0
        keep_searching = True
        core = ""
        while keep_searching:
            try:
                core = _winreg.EnumKey(key, index)
                if core == 'CORE IMPACT Server':
                    core = ""
                    continue
                if "CORE" in core:
                    keep_searching = False
                    break
            except:
                keep_searching = False
        _winreg.CloseKey(key)
        return r"%s\%s" % (subkey, core)



    @staticmethod
    def set_quality_report(enabled=True, dumptype=2):
        """Enabled: True or False
           Dumptype: 0,1,2 (from less detailed to more detailed)"""
        if dumptype not in [0,1,2]:
            dumptype = 2
        if enabled:
            ImpactConfigurationManager._set_registry_value("QualityReport",
                                                      "enabled", "REG_DWORD", 1)
        else:
            ImpactConfigurationManager._set_registry_value("QualityReport",
                                                      "enabled", "REG_DWORD", 0)
        ImpactConfigurationManager._set_registry_value("QualityReport",
                                              "dumptype", "REG_DWORD", dumptype)

    @staticmethod
    def get_impact_bin_path():
        """returns the location of the impact's bin directory"""
        return ImpactConfigurationManager.get_registry_value('paths', 'Binary')

    @staticmethod
    def get_vulndb_dsn():
        """Reutns the Vulns Database OBDC's DSN."""
        return ImpactConfigurationManager.get_registry_value('', 'VulnsDBDSN')

    @staticmethod
    def add_license(lic_path):
        """Adds the given license to Impact. @lic_path is a string containing
        the path to the .lic file."""
        from impact_manager.impact_com_manager import ImpactComManager
        fd = open(lic_path, "r")
        lic = fd.read()
        icm = ImpactComManager()
        lic_mgr = icm.get_license_mgr_com()
        lic_mgr.addLicense(lic)

    @staticmethod
    def set_agent_reconnection_policy(use_reconnection_policy=0,
                                      connection_type="Connect From",
                                      retry_timeout=86400,
                                      retry_interval=30,
                                      port=0):
        """DEPRECATED"""
        import warnings
        from exceptions import DeprecationWarning
        warnings.warn("set_agent_reconnection_policy is DEPRECATED use "
        "ImpactConfigurationManager.AgentsConfiguration.set_reconnection_policy"
        "(use_reconnection_policy, reconnection_timeout, reconnection_interval,"
        " port) instead", DeprecationWarning)

        ImpactConfigurationManager.AgentsConfiguration.set_reconnection_policy(
                                use_reconnection_policy=use_reconnection_policy,
                                reconnection_interval=retry_interval,
                                reconnection_timeout=retry_timeout,
                                port=port)

    @staticmethod
    def set_debug_options(debug_mode='on', detail_level='high'):
        """DEPRECATED"""
        import warnings
        from exceptions import DeprecationWarning
        warnings.warn("set_debug_options is DEPRECATED use instead "
        "ImpactConfigurationManager.AgentsConfiguration.set_module_execution "
        "and ImpactConfigurationManager.ModulesConfiguration.set_module_log",
        DeprecationWarning)

        dmode={'on':(1,0), 'attach':(1,1), 'off':(0,0)}

        ImpactConfigurationManager.ModulesConfiguration.set_module_execution(
                                auto_select_new_tasks=None,
                                allow_dropping_on_unsupported_targets=None,
                                show_system_modules=None,
                                debugging_mode=dmode.get(debug_mode, (0,0))[0],
                                attach_debugger=dmode.get(debug_mode, (0,0))[1],
                                max_concurrency=None)

        ImpactConfigurationManager.ModulesConfiguration.set_module_log(
                                                    detail_level=detail_level,
                                                    add_current_time=None)

    @staticmethod
    def set_search_engines(search_engine, api_id):
        """DEPRECATED"""
        import warnings
        from exceptions import DeprecationWarning
        warnings.warn("set_search_engines is DEPRECATED use "
        "ImpactConfigurationManager.SearchEnginesConfiguration."
        "set_search_engine(search_engine, api_id) instead", DeprecationWarning)
        ImpactConfigurationManager.SearchEnginesConfiguration.set_search_engine(
                                                          search_engine, api_id)

    @staticmethod
    def get_registry_value(sub_path, key):
        """Returns the value of the given key and Impact's registry subpath
        E.g. to get the stats server url, you should do:
        ImpactConfigurationManager.get_registry_value('Stats', 'StatsServerURL')
        E.g. 2: to get the product version, you should do:
        ImpactConfigurationManager.get_registry_value('', 'Version')
        """
        subkey = ImpactConfigurationManager._get_full_path(sub_path)

        regkey = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, subkey)
        try:
            ret = _winreg.QueryValueEx(regkey, key)[0]
        finally:
            _winreg.CloseKey(regkey)
        return ret

    @staticmethod
    def _set_registry_value(sub_path, name, vtype, value):
        try:
            path = ImpactConfigurationManager._get_full_path(sub_path)
            key_type = _winreg.REG_SZ
            if vtype is "REG_DWORD":
                key_type = _winreg.REG_DWORD
            key = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, path,0, 
                                  _winreg.KEY_SET_VALUE)
            try:
                _winreg.SetValueEx(key, name, 0, key_type , value)
            finally:
                _winreg.CloseKey(key)
            return True
        except Exception, e:
            print e
            return False

    @staticmethod
    def _get_full_path(sub_path):
        reg_path = ImpactConfigurationManager.get_registry_path()
        if sub_path == '':
            return reg_path
        elif sub_path[0] == '\\':
            return reg_path + sub_path
        else:
            return reg_path + "\\" + sub_path


    @staticmethod
    def _delete_value(sub_key, value):
        try:
            subkey = ImpactConfigurationManager._get_full_path(sub_key)
            regkey = _winreg.OpenKey(_winreg.HKEY_LOCAL_MACHINE, subkey)
            try:
                _winreg.DeleteValue(regkey, value)
            finally:
                _winreg.CloseKey(regkey)
            return True
        except:
            return False

    class ModulesConfiguration:

        @staticmethod
        def set_module_execution(auto_select_new_tasks='all',
                                 allow_dropping_on_unsupported_targets=0,
                                 show_system_modules=0,
                                 debugging_mode=0,
                                 attach_debugger=0,
                                 max_concurrency=42):
            """Sets the Module Execution settings in the impact's registry.
            auto_select_new_tasks: one of 'disabled', 'main', 'all'
            allow_dropping_on_unsupported_targets: 0 or 1
            show_system_modules: 0 or 1
            debugging_mode: 0 or 1
            attach_debugger: 0 or 1 (ignored if debugging_mode is not 1)
            max_concurrency: a positive integer
            Use None to not modify the value"""

            auto_select = {'disabled':0, 'all':1, 'main':2}
            if auto_select_new_tasks in auto_select:
                ImpactConfigurationManager._set_registry_value(
                                "GUI\ExecutedModulesView",
                                "AutoSelectNewTasks",
                                "REG_DWORD", auto_select[auto_select_new_tasks])

            if allow_dropping_on_unsupported_targets in [0, 1]:
                ImpactConfigurationManager._set_registry_value(
                                          "GUI\Visibility View",
                                          "AllowUnsupportedTargets",
                                          "REG_DWORD",
                                          allow_dropping_on_unsupported_targets)

            if show_system_modules in [0, 1]:
                ImpactConfigurationManager._set_registry_value(
                                "GUI\ExecutedModulesView",
                                "ShowSystemModules",
                                "REG_DWORD", show_system_modules)

            if debugging_mode in [0, 1]:
                val = 0
                if debugging_mode and attach_debugger == 1: val = 3
                elif debugging_mode: val = 2
                ImpactConfigurationManager._set_registry_value(
                                                            "ModuleExecution",
                                                            "DebuggingValue",
                                                            "REG_DWORD", val)

            if isinstance(max_concurrency, int) and max_concurrency > 0:
                ImpactConfigurationManager._set_registry_value(
                                                            "ModuleExecution",
                                                            "MaxTasks",
                                                            "REG_DWORD",
                                                            max_concurrency)

        @staticmethod
        def set_module_log(detail_level='medium', add_current_time=0):
            """Sets the Module Execution settings in the impact's registry.
            detail_level: one of 'low', 'medium', 'high'
            add_current_time: 0 or 1"""

            detail_levels = {'high':2, 'medium':1, 'low':0}
            if detail_level in detail_levels:
                ImpactConfigurationManager._set_registry_value(
                                       "ModuleExecution", "LogDetailLevel",
                                       "REG_DWORD", detail_levels[detail_level])

            if add_current_time in [0, 1]:
                ImpactConfigurationManager._set_registry_value(
                                               "ModuleExecution", "TimeInLines",
                                               "REG_DWORD", add_current_time)

    class AgentsConfiguration:

        @staticmethod
        def set_reconnection_policy(use_reconnection_policy=0,
                                      reconnection_interval=30,
                                      reconnection_timeout=86400,
                                      port=0):
            """Sets the Agent Reconection Policies in the impact's registry.
            use_reconnection_policy should be 1 or 0
            reconnection_interval should be an integer (seconds)
            reconnection_timeout should be an integer (seconds)
            port should be an integer.
            Use None to not modify the value"""

            subpath = r"Entities\Agent Reconnection Policy"
            if use_reconnection_policy in [0,1]:
                ImpactConfigurationManager._set_registry_value(subpath,
                                           "UseReconnectionPolicy",
                                           "REG_DWORD", use_reconnection_policy)

            if isinstance(reconnection_interval, int):
                ImpactConfigurationManager._set_registry_value(subpath,
                           "Retry interval", "REG_DWORD", reconnection_interval)

            if isinstance(reconnection_timeout, int):
                ImpactConfigurationManager._set_registry_value(subpath,
                             "Retry timeout", "REG_DWORD", reconnection_timeout)

            if isinstance(port, int):
                ImpactConfigurationManager._set_registry_value(subpath,
                                                      "Port", "REG_DWORD", port)

        @staticmethod
        def get_reconnection_policy():
            """Returns a 4 value tuple with this info:
            use reconnection policy: 0 or 1
            reconnection interval: <int> seconds
            reconnection timeout: <int> seconds
            port: <int>"""
            subpath = r"Entities\Agent Reconnection Policy"
            try:
                use_reconnect = ImpactConfigurationManager.get_registry_value(
                                               subpath, "UseReconnectionPolicy")
            except:
                use_reconnect = 0
            try:
                interval = ImpactConfigurationManager.get_registry_value(
                                                      subpath, "Retry interval")
            except:
                interval = 30
            try:
                timeout = ImpactConfigurationManager.get_registry_value(subpath,
                                                        "Retry timeout")
            except:
                timeout = 0
            try:
                port = ImpactConfigurationManager.get_registry_value(subpath,
                                                                         "port")
            except:
                port = 0
            return use_reconnect, interval, timeout, port

        @staticmethod
        def set_keep_alive_policy(use_keep_alive=1,
                                      keep_alive_interval=60,
                                      ):
            """Sets the Agent Keep Alive Policies in the impact's registry.
            use_keep_alive should be 1 or 0
            keep_alive_interval should be an integer (seconds)
            Use None to not modify the value"""

            subpath = r"Entities\Agent Keep Alive"
            if use_keep_alive in [0,1]:
                ImpactConfigurationManager._set_registry_value(subpath,
                                           "KeepAliveEnabled",
                                           "REG_DWORD", use_keep_alive)

            if isinstance(keep_alive_interval, int):
                ImpactConfigurationManager._set_registry_value(subpath,
                                                       "AgentKeepAliveInterval", 
                                                       "REG_DWORD", 
                                                       keep_alive_interval)

        @staticmethod
        def get_keep_alive_policy():
            """Returns a 2 value tuple with this info:
            use keep alive policy: 0 or 1
            keep alive interval: <int> seconds"""
            
            subpath = r"Entities\Agent Keep Alive"
            try:
                use_keep_alive = ImpactConfigurationManager.get_registry_value(
                                               subpath, "KeepAliveEnabled")
            except:
                use_keep_alive = 1
            try:
                interval = ImpactConfigurationManager.get_registry_value(
                                              subpath, "AgentKeepAliveInterval")
            except:
                interval = 60
            
            return use_keep_alive, interval

    class OneStepRPTConfiguration:

        @staticmethod
        def set_outgoing_smtp_srver(address="", port=25):
            """Set the Outgoing SMTP Server settings.
            address should be a string
            port should be an integer
            Use None to not modify the value"""

            if isinstance(address, basestring):
                ImpactConfigurationManager._set_registry_value(
             "AdvancedSettings", "OutgoingSMTPAddressServer", "REG_SZ", address)

            if isinstance(port, int):
                ImpactConfigurationManager._set_registry_value(
               "AdvancedSettings", "OutgoingSMTPAddressPort", "REG_DWORD", port)


        @staticmethod
        def get_outgoing_smtp_srver():
            """Returns (address<str>, port<int>) with the current settings"""
            try:
                address = ImpactConfigurationManager.get_registry_value(
                                "AdvancedSettings", "OutgoingSMTPAddressServer")
            except:
                address = ""
            try:
                port = ImpactConfigurationManager.get_registry_value(
                                  "AdvancedSettings", "OutgoingSMTPAddressPort")
            except:
                port = 25
            return address, port

    class SearchEnginesConfiguration:

        @staticmethod
        def set_search_engine(search_engine, api_id):
            """Sets a search engine ip id.
            search_engine should be either 'Yahoo', or 'Bing'"""
            if search_engine in ['Yahoo','Bing']:
                ImpactConfigurationManager._set_registry_value(
                                                    r"ClientSide\SearchEngines",
                                           search_engine, "REG_SZ", str(api_id))
        @staticmethod
        def get_search_engines():
            """Returns a dictionary where keys are Search Engine names and
            values its corresponding API ids"""
            try:
                yahoo = ImpactConfigurationManager.get_registry_value(
                                           r"ClientSide\SearchEngines", "Yahoo")
            except:
                yahoo = None
            try:
                bing = ImpactConfigurationManager.get_registry_value(
                                            r"ClientSide\SearchEngines", "Bing")
            except:
                bing = None
            return {'Yahoo':yahoo, 'Bing':bing}