#--
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--

import time

#import comtypes.client
import win32com.client
import _winreg

from impact_manager.impact_configuration_manager \
                                               import ImpactConfigurationManager

from impact_manager.resources.singleton import Singleton
from impact_manager.resources.resultset import ResultSet
from impact_manager.resources.wservice import WService


class ImpactDBManager:
    __metaclass__ = Singleton

    def __init__(self):
        """This is a singleton object. The first call will create the instance
        the following calls will just return the same instance"""
        self.__initialize()

    def execute(self, query, use_vuln_db = False):
        """ Runs the given query against the Impact's database unless
        the use_vuln_db flag is set to True in which case the query will
        run against the corevuln database.
        Returns a resulset instance of lib.util.resultset.Resultset"""
        self.check_connection_is_opened()
        if use_vuln_db:
            ado_rs = win32com.client.Dispatch(r'ADODB.Recordset')
            ado_rs.Open(query, self.__conn_vuln, 1 ,3)
        else:
            ado_rs = win32com.client.Dispatch(r'ADODB.Recordset')
            ado_rs.Open(query, self.__conn, 1 ,3)

        rs = ResultSet(ado_rs)
        return rs

    def execute_non_select(self, query, use_vuln_db = False):
        """ Use this instead of 'execute' for a query that won't return results
        (e.g. UPDATE, INSERT, DELETE """
        self.check_connection_is_opened()
        if use_vuln_db:
            self.__conn_vuln.Execute(query)
        else:
            self.__conn.Execute(query)

    @staticmethod
    def addslashes(s):
        """Returns a string with backslashes before characters that need to be
        quoted in database queries etc. These characters are single quote ('),
        double quote ("), backslash (\) and NUL (the NULL byte). """
        s = str(s)
        return s.replace("\0", "").replace("\\", "\\\\").replace("'", "''"
                                                           ).replace('"', '\\"')


    def start_service(self):
        """ Attemps to start the SQLSERVER service. Returns True if it is
        running (whether it was started or it was already running). Otherwise
        returns False """
        try:
            sqlserver = WService(self.__service_name)
            if sqlserver.status() is not "RUNNING":
                sqlserver.start()
                #wait up to 20 seconds for the service to start
                result = sqlserver.fetchstatus("RUNNING", 20)
                if result is "TIMEDOUT":
                    return False
        except:
            return False
        return True

    def stop_service(self):
        """ Attemps to stop the SQLSERVER service. Returns True if it is not
        running (whether it was stopped or it was already shutted down).
        Otherwise returns False """
        try:
            sqlserver = WService(self.__service_name)
            if sqlserver.status() is not "STOPPED":
                sqlserver.stop()
                #wait up to 20 seconds for the service to stop
                result = sqlserver.fetchstatus("STOPPED", 20)
                if result is "TIMEDOUT":
                    return False
        except Exception, e:
            print e
            return False
        return True

    def check_connection_is_opened(self):
        """Checks if the connection with the database is opened if not, attempts
        to restart it"""
        try:
            self.__conn.Execute("SELECT 1;")
        except:
            try:
                self.__conn.Close()
                self.__conn_vuln.Close()
            except:
                pass
            self.__initialize()

    @staticmethod
    def get_connection_string():
        """Returns the impact's database connection string"""
        return ImpactConfigurationManager.get_registry_value('',
                                                           'DBConnectionString')

    def __initialize(self):
        #self.__conn = comtypes.client.CreateObject('ADODB.Connection')
        self.__conn = win32com.client.Dispatch('ADODB.Connection')
        self.__conn.ConnectionString = self.get_connection_string()

        #second connection for the vulnerability DB
        self.__conn_vuln = win32com.client.Dispatch('ADODB.Connection')

        self.__service_name = "MSSQL$IMPACT"
        self.start_service()

        #start both connections
        self.__conn.Open()
        self.__conn_vuln.Open(ImpactConfigurationManager.get_vulndb_dsn())
