#--
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--
import os
import string
import random
from xml.dom import minidom


from impact_manager.resources.module_execution_event_handler \
                                 import UpdateManagerModuleExecutionEventHandler
from impact_manager.impact_lib_exceptions import ImpactModuleException
from impact_manager.impact_com_manager import ImpactComManager
from impact_manager.workspace_manager import WorkspaceManager
from impact_manager.entities.module import Module

class ImpactUpdateManager:

    def __init__(self):
        """ Handles general updates operation in Impact.
        All the methods in this class are static.
        You dont need an instance."""
        pass

    @staticmethod
    def check_updates(timeout=180):
        """Returns a structure with pending updates information"""
        ## Currently the COM's checkUpdate is not raising client calls
        #requesting for the Proxy configuration. So the following code is
        #commented out until this is fixed. Meanwhile we use an alternate
        #method just as the GUI does

        #icm = ImpactComManager()
        #update_com = icm.get_update_mgr_com()
        #eh = UpdateManagerModuleExecutionEventHandler()
        #update_com.checkUpdates(eh.wrap_me())
        #return eh.check_updates_status()

        ret = []
        doc = None
        fname = ''.join( random.Random().sample(string.letters+string.digits,
                                                                   10)) + '.tmp'
        fname = os.path.abspath(os.path.join(os.path.dirname(__file__), fname))

        #workspace to use should be 'Console Modules Helper', however it seems
        #there's a race condition that makes impact crash if impact
        #attempts to delete the workspace (as periodically done) but it is
        #open by impact_manager. So I'll create instead an
        #interla 'Impact Manager Modules Helper'
        workspace_name = "Impact Manager Modules Helper"

        w = WorkspaceManager.open_or_create_workspace(workspace_name, '')
        w.set_property("INTERNAL", 1)
        m = Module(w, "Create Available and Installed Packages File")
        m.add_parameter("FILE", fname)

        try:
            m.sync_run(timeout=timeout)
            doc = minidom.parse(fname)
        except ImpactModuleException:
            try:
                m.stop()
            except:
                pass
            raise
        finally:
            try:
                os.remove(fname)
            except:
                pass
        pkgs = doc.getElementsByTagName('package_available')
        for pkg in pkgs:
            item = {'category':None,
                    'cve':None,
                    'name':None,
                    'date':None,
                    'description':None}
            if pkg.hasAttribute('category'):
                item['category'] = pkg.getAttribute('category')
            if pkg.hasAttribute('cve'):
                item['cve'] = pkg.getAttribute('cve')
            if pkg.hasAttribute('name'):
                item['name'] = pkg.getAttribute('name')
            if pkg.hasAttribute('date'):
                item['date'] = pkg.getAttribute('date')

            for child in pkg.childNodes:
                if child.nodeName == 'description':
                    desc = ''
                    for t in child.childNodes:
                        if t.nodeType in [doc.TEXT_NODE,doc.CDATA_SECTION_NODE]:
                            desc += t.nodeValue
                    item['description'] = desc
                    break
            ret.append(item)

        return ret

    @staticmethod
    def get_updates():
        """Installs pending updates in impact, returns True if the process
        finished correctly, False otherwise"""
        icm = ImpactComManager()
        update_com = icm.get_update_mgr_com()
        eh = UpdateManagerModuleExecutionEventHandler()
        update_com.getUpdates(eh.wrap_me())

        return eh.get_updates_status()
