#--
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--

import sys
import os
import inspect

import win32file
import win32process

from impact_manager.impact_configuration_manager \
                                               import ImpactConfigurationManager

class CustomEventHandlerFactory(object):
    @staticmethod
    def get_event_handler(event_type, more_in, more_out):
        params = {}
        out = {}
        for mi in more_in:
            for item in mi:
                params[str(item.Key)] = str(item.Value)
        try:
            for mo in more_out:
                for item in mo:
                    for subitem in item:
                        out[str(subitem.Key)] = str(subitem.Value)
        except:
            pass
            #TODO: it seems that more_out has a second item which raises
            #a memory error when readed, i'm overlooking that error here
            #apparently nothing is lost

        for k, v in sys.modules[__name__].__dict__.items():
            if k == event_type and inspect.isclass(v) \
            and issubclass(v, CustomEventHandler) and v != CustomEventHandler:
                return v(params, out)
        msg = "There's a custom event which has not be implemented. " \
        "Please ask Seba:" \
        "\n Event Type: %s"\
        "\n More In: %s" \
        "\n More Out: %s" % (event_type, params, out)
        print msg
        raise NotImplementedError(msg)


class CustomEventHandler(object):
    def __init__(self, params, out):
        #Find params names for each custom event at:
        #http://silo.servers.core-sdi.com/projects/sniper/browser/components/
        #kernel/trunk/interface/impact_client/ModuleExecutionEventHandlerEnums.h

        #Find impact GUI implementation of each event at:
        #http://silo.servers.core-sdi.com/projects/sniper/browser/components/
        #kernel/trunk/interface/impact_client/module_execution.cpp

        self.params = params
        self._handle_return = out
        self._extra_return = None
        self._handled = False

    def handle(self):
        self._handle_implementation()
        return self._format_handle_return(), self._extra_return

    def _format_handle_return(self):
        return self._handle_return.items()

    def _handle_implementation(self):
        raise NotImplementedError("Custom event handles must implement "
                                  "handle_event method.")


class CC_MSG_BOX(CustomEventHandler):
    def _handle_implementation(self):

        ##was this handled already?
        ##if so, let the first handler take care of the result...
        if "CC_MSG_RESULT" in self._handle_return:
            return

        ##not implemented by default, provide an invalid return value (0) to
        #indicate so
        self._handle_return[u'CC_MSG_RESULT'] = u"0"


class CC_IE_SETTINGS(CustomEventHandler):

    def _handle_implementation(self):
        #Keys to return:
        #CC_IE_SETTINGS_PROXIES: IP and optionally port of the proxy server
        #CC_IE_SETTINGS_BYPASS: Bypass items list separated by semicolons

        ##was this handled already?
        if "CC_IE_SETTINGS_PROXIES" in self._handle_return:
            return

        self._handle_return[u'CC_IE_SETTINGS_PROXIES'] = u''
        self._handle_return[u'CC_IE_SETTINGS_BYPASS'] = u''

        import _winreg

        reg_path =r"Software\Microsoft\Windows\CurrentVersion\Internet Settings"

        regkey = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, reg_path)
        try:
            k1 = _winreg.QueryValueEx(regkey, 'ProxyEnable')[0]
            k2 = _winreg.QueryValueEx(regkey, 'ProxyServer')[0]
            k3 = _winreg.QueryValueEx(regkey, 'ProxyOverride')[0]
        finally:
            _winreg.CloseKey(regkey)
        if k1 == 1:
            self._handle_return[u'CC_IE_SETTINGS_PROXIES'] = k2
            self._handle_return[u'CC_IE_SETTINGS_BYPASS'] = k3

class CC_EXECUTE_CMD(CustomEventHandler):

    def _handle_implementation(self):
        #Input:
        # CC_CMD_LINE: path to console server
        #Keys to return:
        #ProcessId: ConsoleServer's pid
        #ThreadId: ConsoleServer's main window thread id

        ##was this handled already?
        if "ProcessId" in self._handle_return:
            return

        cur_dir = os.getcwd()

        startup_info = win32process.STARTUPINFO()
        cmd = self.params['CC_CMD_LINE']
        working_directory = ImpactConfigurationManager.get_impact_bin_path()

        try:
            os.chdir(working_directory)
            hProcess, hThread, dwPid, dwTid = win32process.CreateProcess(None,
                                                  cmd,
                                                  None,
                                                  None,
                                                  0,
                                                  win32process.DETACHED_PROCESS,
                                                  None,
                                                  working_directory,
                                                  startup_info)
        finally:
            os.chdir(cur_dir)

        win32file.CloseHandle(hProcess)
        win32file.CloseHandle(hThread)

        self._handle_return[u'ProcessId'] = unicode(dwPid)
        self._handle_return[u'ThreadId'] = unicode(dwTid)


class FileBrowserModuleEvent(CustomEventHandler):
    def _handle_implementation(self):
        ##was this handled already?
        if "FileBrowserModuleEvent_Result" in self._handle_return:
            return

        self._handle_return['FileBrowserModuleEvent_Result'] = u'0'
