#--
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--

import time
import comtypes

from comtypes.gen import impact_core_comLib as impact_core_com
from impact_manager.impact_db_manager import ImpactDBManager
from impact_manager.resources.custom_event_handlers \
                                                import CustomEventHandlerFactory

class GeneralModuleExecutionEventHandler(comtypes.COMObject):
    _com_interfaces_ = [impact_core_com.IModuleExecutionEventHandler]

    #EVENT TYPES
    OUTPUT = 1
    LOGLINE = 2
    PROGRESS_UPDATED = 3
    TASK_INITIAL = 4
    TASK_RUNNING = 5
    TASK_FINISHED = 6
    TASK_ABORTED = 7
    TASK_STOPPED = 8
    TASK_STOPPING = 9
    CHECK_UPDATES_FINAL = 10
    TASK_ERROR = 11
    CUSTOM = 12


    def __init__(self):
        self.__listeners = {GeneralModuleExecutionEventHandler.OUTPUT:[],
                    GeneralModuleExecutionEventHandler.LOGLINE:[],
                    GeneralModuleExecutionEventHandler.PROGRESS_UPDATED:[],
                    GeneralModuleExecutionEventHandler.TASK_INITIAL:[],
                    GeneralModuleExecutionEventHandler.TASK_RUNNING:[],
                    GeneralModuleExecutionEventHandler.TASK_FINISHED:[],
                    GeneralModuleExecutionEventHandler.TASK_ABORTED:[],
                    GeneralModuleExecutionEventHandler.TASK_STOPPED:[],
                    GeneralModuleExecutionEventHandler.TASK_STOPPING:[],
                    GeneralModuleExecutionEventHandler.CHECK_UPDATES_FINAL:[],
                    GeneralModuleExecutionEventHandler.TASK_ERROR:[],
                    GeneralModuleExecutionEventHandler.CUSTOM:[]}

    def wrap_me(self):
        """"Returns a wrapper of this interface to be passed as argument to the
        COM function"""
        return self.QueryInterface(impact_core_com.IModuleExecutionEventHandler)

    def subscribe(self, event_types, callback_fc, *callback_additional_params):
        """Subscribes a function to the required event_types"""
        listener = {}
        listener['callback_fc'] = callback_fc
        listener['callback_params'] = callback_additional_params

        for event_type in event_types:
            if event_type in self.__listeners:
                self.__listeners[event_type].append(listener)


    def IModuleExecutionEventHandler_onCustomEvent(self, workspace_id, task_id,
                                                 event_type, more_in, more_out):
        ce_handler = CustomEventHandlerFactory.get_event_handler(event_type,
                                                              more_in, more_out)
        a, b = ce_handler.handle()

        return a


    def IModuleExecutionEventHandler_onEvent(self, event_type, workspace_id,
                                                                 task_id, more):
        data_dict = {'EVENT_TIME': time.time(),
                     'EVENT_TYPE':event_type,
                     'WORKSPACE_ID':workspace_id,
                     'TASK_ID':task_id
                    }
        for item in more:
            for subitem in item:
                data_dict[str(subitem.Key)] = str(subitem.Value)

        for l in self.__listeners[event_type]:
            try:
                callf = l['callback_fc']
                callparams = l['callback_params'][0].__add__((data_dict,))
                callf(*callparams)
            except Exception, e:
                #TODO: what should we do if the function call, or the function
                #itself fails? Is not our fault, but the listener's. Anyway
                #we should let him know somehow
                print "[GeneralModuleExecutionEventHandler]Error in callback " \
                      "subscriber:", e