#--
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--

class ResultSet:
    def __init__(self, ado_rs):
        """Creates a persistent resultset from an ADODB resultset"""
        self.__ado_rs = ado_rs

        self.__cursor = -1
        self.__field_names = []

        self.EOF = self.__ado_rs.EOF
        for i in range (self.__ado_rs.Fields.Count):
            self.__field_names.append(self.__ado_rs.Fields.Item(i).Name)


    def num_rows(self):
        """Returns the record count of this resultset"""
        return self.__ado_rs.RecordCount

    def num_fields(self):
        """Returns the columns count of this resultset"""
        return self.__ado_rs.Fields.Count

    def value(self, index_or_key):
        """Returns the value for the current row's column by index or key"""
        if not self.num_rows():
            return None
        index = -1
        if isinstance(index_or_key, int):
            index = index_or_key
        elif isinstance(index_or_key, basestring):
            try:
                index = self.__field_names.index(index_or_key)
            except:
                index = -1
        if index < 0 or index >= self.num_fields():
            return None
        return self.__ado_rs.Fields.Item(index).value


    def get_row_as_list(self):
        """Returns the current row as a list [value_col_1, value_col_2, ...]"""
        return [self.__ado_rs.Fields.Item(i).value
                for i in range(self.num_fields())]

    def get_row_as_dict(self):
        """Returns the current row as a dict {'colum_name':value, ...}"""
        return dict([(self.__field_names[i],self.__ado_rs.Fields.Item(i).value)
                     for i in range(self.num_fields())])

    def has_key(self, _str):
        """Returns True if a column name matches the key provided """
        return _str in self.__field_names

    def next(self):
        """Moves the cursor to the next position"""
        if not self.__ado_rs.EOF:
            self.__ado_rs.MoveNext()
            self.EOF = self.__ado_rs.EOF


    def restart_cursor(self):
        """Resets the cursor to the position 0"""
        if self.num_rows():
            self.__ado_rs.MoveFirst()
            self.EOF = self.__ado_rs.EOF