#--
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2012 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--

import random
import os
import time
import shutil

from impact_manager.impact_configuration_manager \
                                               import ImpactConfigurationManager
from impact_manager.impact_com_manager import ImpactComManager
from impact_manager.impact_db_manager import ImpactDBManager
from impact_manager.entities.workspace import Workspace
from impact_manager.resources.resultset import ResultSet
from impact_manager.impact_lib_exceptions import ImpactWorkspaceException

class WorkspaceManager:
    __impact_com = ImpactComManager()
    __db = ImpactDBManager()

    @staticmethod
    def open_workspace(name, passphrase="11111111"):
        """ Opens a workspace with the given name and passphrase. If not
        passphrase is provided then '11111111' is used. Returns an instance of
        impact_manager.Workspace, Throws an ImpactWorkspaceException if
        anything goes wrong."""
        try:
            wsmgr = WorkspaceManager.__impact_com.get_workspace_mgr_com()
            ws = wsmgr.openWorkspace(int(WorkspaceManager.get_workspace_id(name)
                                         ), passphrase)
            ws_entity = Workspace(ws.getId(), name, passphrase, ws)
            return ws_entity
        except Exception, e:
            raise ImpactWorkspaceException("Error opening workspace %s: %s" %
                                           (name, e.message))

    @staticmethod
    def open_workspace_by_id(id, passphrase="11111111"):
        """ Opens a workspace with the given id and passphrase. If not
        passphrase is provided then '11111111' is used. Returns an instance of
        impact_manager.Workspace, Throws an ImpactWorkspaceException if
        anything goes wrong."""
        try:
            wsmgr = WorkspaceManager.__impact_com.get_workspace_mgr_com()
            ws = wsmgr.openWorkspace(int(id), passphrase)
            ws_entity = Workspace(ws.getId(), "", passphrase, ws)
            return ws_entity
        except Exception, e:
            raise ImpactWorkspaceException(
                     "Error opening workspace with id %d: %s" % (id, e.message))

    @staticmethod
    def delete_workspace(name):
        """ Deletes a workspace named as the provided parameter. Returns True
        if a workspace was deleted """
        try:
            wsmgr = WorkspaceManager.__impact_com.get_workspace_mgr_com()
            wsmgr.deleteWorkspace(int(WorkspaceManager.get_workspace_id(name)))
            return True
        except:
            return False

    @staticmethod
    def delete_workspace_by_id(workspace_id):
        """ Deletes the workspace with the given ID. Returns True if a workspace
        was deleted """
        try:
            wsmgr = WorkspaceManager.__impact_com.get_workspace_mgr_com()
            wsmgr.deleteWorkspace(int(workspace_id))
            return True
        except:
            return False

    @staticmethod
    def create_workspace(name, passphrase="11111111"):
        """ Creates a workspace with the given name and passphrase. If not
        passphrase is provided then '11111111' is used. Returns an instance of
        impact_manager.Workspace, Throws an ImpactWorkspaceException if anything
        goes wrong."""

        try:
            wsmgr = WorkspaceManager.__impact_com.get_workspace_mgr_com()
            seed = []
            for i in range(256):
                seed.append(random.randint(0,1))
            lic = WorkspaceManager.__impact_com.get_license_mgr_com(
                                                          ).getGeneralLicense()
            not_visible = 0
            ws = wsmgr.createWorkspace(name, passphrase, seed, lic, not_visible)
            ws_entity = Workspace(ws.getId(), name, passphrase, ws)
            ws_entity.set_property("CLIENT_COMPANY","")
            ws_entity.set_property("CONTACT_EMAIL","")
            ws_entity.set_property("CONTACT_NAME","")
            ws_entity.set_property("CONTACT_PHONE","")
            return ws_entity
        except Exception, e:
            raise ImpactWorkspaceException("Error creating workspace %s: %s"
                                           % (name, e.message))

    @staticmethod
    def open_or_create_workspace(name, passphrase="11111111"):
        """ Attemps to open a workspace with the given name and passphrase,
        if it not exists attempts to create it. Returns an instance of
        impact_manager.Workspace, throws an ImpactWorkspaceException if anything
        goes wrong. """
        try: #try to open it
            ws = WorkspaceManager.open_workspace(name, passphrase)
        except: #if not possible, then try to create it
            ws = WorkspaceManager.create_workspace(name, passphrase)
        return ws

    @staticmethod
    def import_workspace(path_src_db, workspace_name):
        """Imports (and replaces if exists) the workspace workspace_name from
        the database in path_src_db into the impact's database """

        def check_db_files(path):
            try:
                os.stat(path)
            except Exception, e:
                raise Exception("Unable to access '%s': %s" % (path_src_db, e))

        def copy_to_mssql_data(src):
            try:
                #get mssql data directory
                limit_l = "attachdbfilename="
                limit_r = ";"
                con_str = ImpactDBManager.get_connection_string().lower()
                temp = con_str[con_str.index(limit_l) + len(limit_l):]
                data_dir = os.path.dirname(temp[:temp.index(limit_r)])
                #generate temp file name
                file_name = "temp_%s.mdf" % (time.strftime("%Y%m%d%H%M%S"))
                #copy the db file to the data directory
                dst = os.path.join(data_dir, file_name)
                shutil.copy(src, dst)
                #attempt to copy log files as well
                try:
                    shutil.copy(src.replace(".mdf","_log.ldf"),
                                dst.replace(".mdf", "_log.ldf"))
                except:
                    pass
                return dst
            except Exception, e:
                raise Exception( "Unable to generate temp database file:%s"
                                 % (e) )

        def get_src_connection_string(src_file, db_name):
            conn_str = ImpactDBManager.get_connection_string()
            tokens = conn_str.split(';')
            src_str = ""
            for t in tokens:
                k,v = t.split('=')
                if k == 'AttachDBFilename':
                    v = src_file
                elif k == 'Database':
                    v = db_name
                src_str += k + "=" + v + ";"
            return src_str

        def get_src_workspace_id(src_conn_str, workspace_name):
            import win32com.client
            conn = win32com.client.Dispatch('ADODB.Connection')
            conn.ConnectionString = src_conn_str
            conn.Open()
            workspace_name = workspace_name.replace("\\",
                                                    "\\\\").replace("'","\\'")
            query = "SELECT Workspace_Id FROM TWorkspace WHERE Name = '%s'" % (
                                                                 workspace_name)
            ado_rs , _ = conn.Execute(query)
            rs = ResultSet(ado_rs)
            conn.Execute("USE master;")
            ret = rs.value(0)
            ado_rs.Close()
            conn.Close()
            del conn
            del ado_rs
            if ret is None:
                raise Exception(
                            "workspace '%s' does not exists in source database."
                            %(workspace_name))
            return ret

        def get_importer_exe():
            importer = ImpactConfigurationManager.get_impact_bin_path() \
                        + r"\importexport.exe"
            return importer

        def create_bat_file(src_str, dst_str, importerexe, w_id):
            temp = "temp_%s.bat" % (time.strftime("%Y%m%d-%H%M%S"))
            temp = os.path.abspath(temp)
            f = open(temp, "w")

            cmd = '"%s" "%s" "%s"' % (importerexe, src_str, dst_str)
            f.write(cmd + "\n")
            f.write("exit" + "\n")
            f.close()

            return temp

        def drop_database(dbname):
            db = ImpactDBManager()
            db.execute("DROP DATABASE %s;" % (dbname))

        passed = True
        error = ""
        try:
            check_db_files(path_src_db)

            new_src_db = copy_to_mssql_data(path_src_db)

            dst_conn_str = ImpactDBManager.get_connection_string()

            new_db_name = "impact_tmp%s" % (time.strftime("%Y%m%d%H%M%S"))

            src_conn_str = get_src_connection_string(new_src_db, new_db_name)

            wsp_id = get_src_workspace_id(src_conn_str, workspace_name)

            importer = get_importer_exe()

            WorkspaceManager.delete_workspace(workspace_name)

            batf = create_bat_file(src_conn_str, dst_conn_str, importer, wsp_id)

            os.system('"%s"' % (batf))
        except Exception, e:
            passed = False
            error = e
        finally:
            try:
                os.remove(batf)
            except:
                pass
            try:
                drop_database(new_db_name)
            except Exception, e:
                #print "could not drop DB:",e
                pass
            try:
                os.remove(new_src_db)
            except:
                pass
        if not passed:
            raise ImpactWorkspaceException("Could not import workspace: %s"
                                           %(error))

    @staticmethod
    def get_workspace_id(name):
        """ get the correspondig ID of the workspace named after 'name'
        Throws an ImpactWorkspaceException if the id can't be obtained"""
        wsmgr = WorkspaceManager.__impact_com.get_workspace_mgr_com()
        ws_id = wsmgr.getWorkspaceId(name)
        if ws_id == -1:
            raise ImpactWorkspaceException("Workspace '%s' does not exist."
                                           % (name))
        return ws_id

    @staticmethod
    def get_com_workspace_manager():
        """ Returns the WorkspaceManger COM object """
        return WorkspaceManager.__impact_com.get_workspace_mgr_com()

    @staticmethod
    def list_workspaces(internal=False, from_scheduler=False):
        """ Returns a list of workspaces names. By default it ommits Internal
        workspaces and workspaces from scheduler. You can provide two flags:
            internal and from_scheduler
        each flag can have three values.
            None: It doesn't matter if the workspaces is internal|from_scheduler
            True: Included only if workspace is internal|from_scheduler
            False: Included only if workspace is not internal|from_scheduler
        For example:
         List workspaces created by the scheduler: list_workspaces(None, True)
         List internal workspaces:                 list_workspaces(True, None)
         List all workspaces:                      list_workspaces(None, None)
         List all user-created workspaces:         list_workspaces(False, False)
                                                   or just list_workspaces() """

        cond1 = ""
        if internal is False:
            cond1 = "AND EXISTS " \
                    "(SELECT p.Workspace_Id " \
                    " FROM TWorkspaceProperty p " \
                    " WHERE p.Workspace_Id = w.Workspace_Id " \
                    " AND p.Name = 'INTERNAL' " \
                    " AND p.Value_Tx = '0')"
        elif internal is True:
            cond1 = "AND EXISTS " \
                    "(SELECT p.Workspace_Id " \
                    " FROM TWorkspaceProperty p " \
                    " WHERE p.Workspace_Id = w.Workspace_Id " \
                    " AND p.Name = 'INTERNAL' " \
                    " AND p.Value_Tx = '1')"
        cond2 = ""
        if from_scheduler is False:
            cond2 = "AND NOT EXISTS " \
                    "(SELECT p.Workspace_Id " \
                    " FROM TWorkspaceProperty p " \
                    " WHERE p.Workspace_Id = w.Workspace_Id " \
                    " AND p.Name = 'FROM_SCHEDULER' " \
                    " AND p.Value_Tx = '1')"
        elif from_scheduler is True:
            cond2 = "AND EXISTS " \
                    "(SELECT p.Workspace_Id " \
                    " FROM TWorkspaceProperty p " \
                    " WHERE p.Workspace_Id = w.Workspace_Id " \
                    " AND p.Name = 'FROM_SCHEDULER' " \
                    " AND p.Value_Tx = '1')"

        query = "SELECT w.Name " \
                "FROM TWorkspace w " \
                "WHERE 1=1 %s %s;" % (cond1, cond2)
        db = ImpactDBManager()
        rs = db.execute(query)
        ret = []
        while not rs.EOF:
            ret.append(rs.value(0))
            rs.next()
        return ret