#!/usr/bin/env python
#--
#
# Author:
#   Sebastian Tello
#
# Copyright (c) 2003,2011 CORE Security Technologies, CORE SDI Inc.
# All rights reserved.
#
# This computer software is owned by Core SDI Inc. and is
# protected by U.S. copyright laws and other laws and by international
# treaties.  This computer software is furnished by CORE SDI Inc.
# pursuant to a written license agreement and may be used, copied,
# transmitted, and stored only in accordance with the terms of such
# license and with the inclusion of the above copyright notice.  This
# computer software or any other copies thereof may not be provided or
# otherwise made available to any other person.
#
#
# THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED. IN NO EVENT SHALL CORE SDI Inc. BE LIABLE
# FOR ANY DIRECT,  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY OR
# CONSEQUENTIAL  DAMAGES RESULTING FROM THE USE OR MISUSE OF
# THIS SOFTWARE
#
#--

__VERSION__ = "0.2.12"

import os, glob, fnmatch
from distutils.core import setup
import distutils.command.install_data


def opj(*args):
    path = os.path.join(*args)
    return os.path.normpath(path)

class smart_install_data(distutils.command.install_data.install_data):
    """need to change self.install_dir to the actual library dir"""
    def run(self):
        install_cmd = self.get_finalized_command('install')
        self.install_dir = getattr(install_cmd, 'install_lib')
        return distutils.command.install_data.install_data.run(self)

def find_data_files(srcdir, *wildcards, **kw):
    badnames=[".pyc","~",".pyo"]
    def walk_helper(arg, dirname, files):
        if '.svn' in dirname:
            return
        names = []
        lst, wildcards = arg
        for wc in wildcards:
            wc_name = opj(dirname, wc)
            for f in files:
                filename = opj(dirname, f)
                if not any(bad in filename for bad in badnames):
                    if (fnmatch.fnmatch(filename, wc_name)
                    and not os.path.isdir(filename)):
                        names.append(filename)
        if names:
            lst.append( (dirname, names ) )

    file_list = []
    recursive = kw.get('recursive', True)
    if recursive:
        os.path.walk(srcdir, walk_helper, (file_list, wildcards))
    else:
        walk_helper((file_list, wildcards),
                    srcdir,
                    [os.path.basename(f) for f in glob.glob(opj(srcdir, '*'))])
    return file_list

if __name__ == "__main__":

    setup(
        name = "impact_manager",
        version = __VERSION__,
        author = "Sebastian Tello",
        author_email = "stello@coresecurity.com",
        description = "Manages Impact Pro through python via COM and SQL Server",
        license = "",
        packages = ['impact_manager'],
        data_files = find_data_files('impact_manager', '*.*'),
        zip_safe = False,
        cmdclass = { 'install_data': smart_install_data },
        long_description="Manages Impact Pro through python via COM and SQL Server"
    )
